"use client";

import type { StripeError } from "@stripe/stripe-js";

import * as React from "react";
import {
    useStripe,
    useElements,
    PaymentElement,
    Elements,
} from "@stripe/react-stripe-js";

import StripeTestCards from "./StripeTestCards";

import getStripe from "../../utils/get-stripe";
import { createPaymentIntent} from "../../actions/stripe";
import {Form} from "antd";
import {Payment} from "../../types/payment";
import {CustomInput} from "../view/CustomInput";
import {i18nText} from "../../i18nKeys";
import {FC, useEffect} from "react";
import {getPersonalData} from "../../actions/profile";

type PaymentFormProps = {
    amount: number,
    sessionId?: string
}


export const CheckoutForm: FC<PaymentFormProps>  = ({amount, sessionId}) => {
    const [input, setInput] = React.useState<{
        paySumm: number;
        cardholderName: string;
    }>({
        paySumm: 1,
        cardholderName: "",
    });
    const [form, ] = Form.useForm<Payment>();
    const formAmount = Form.useWatch('amount', form);
    const [paymentType, setPaymentType] = React.useState<string>("");
    const [payment, setPayment] = React.useState<{
        status: "initial" | "processing" | "error";
    }>({ status: "initial" });
    const [errorMessage, setErrorMessage] = React.useState<string>("");

    const stripe = useStripe();
    const elements = useElements();

    const PaymentStatus = ({ status }: { status: string }) => {
        switch (status) {
            case "processing":
            case "requires_payment_method":
            case "requires_confirmation":
                return <h2>Processing...</h2>;

            case "requires_action":
                return <h2>Authenticating...</h2>;

            case "succeeded":
                return <h2>Payment Succeeded 🥳</h2>;

            case "error":
                return (
                    <>
                        <h2>Error 😭</h2>
                        <p className="error-message">{errorMessage}</p>
                    </>
                );

            default:
                return null;
        }
    };

    useEffect(() => {
        elements?.update({ amount: formAmount * 100 });
    }, [formAmount]);

    const handleInputChange: React.ChangeEventHandler<HTMLInputElement> = (e) => {
        setInput({
            ...input,
            [e.currentTarget.name]: e.currentTarget.value,
        });
    };

    const onSubmit = async (data) => {
        try {
            if (!elements || !stripe) return;

            setPayment({ status: "processing" });

            const { error: submitError } = await elements.submit();

            if (submitError) {
                setPayment({ status: "error" });
                setErrorMessage(submitError.message ?? "An unknown error occurred");

                return;
            }

            // Create a PaymentIntent with the specified amount.
            console.log('DATA', data);
            const { client_secret: clientSecret } = await createPaymentIntent(
                {amount: amount},
            );

            // Use your card Element with other Stripe.js APIs
            const { error: confirmError } = await stripe!.confirmPayment({
                elements,
                clientSecret,
                confirmParams: {
                    return_url: `${window.location.origin}/ru/payment/result`,
                    payment_method_data: {
                        allow_redisplay: 'limited',
                        billing_details: {
                            name: input.cardholderName,
                        },
                    },
                },
            });

            if (confirmError) {
                setPayment({ status: "error" });
                setErrorMessage(confirmError.message ?? "An unknown error occurred");
            }
        } catch (err) {
            const { message } = err as StripeError;

            setPayment({ status: "error" });
            setErrorMessage(message ?? "An unknown error occurred");
        }
    };



    return (
        <>
            <Form form={form} onFinish={onSubmit}>
                <fieldset className="elements-style">
                    <StripeTestCards/>
                    <legend>Your payment details:</legend>
                    {paymentType === "card" ? (
                        <input
                            placeholder="Cardholder name"
                            className="elements-style"
                            type="Text"
                            name="cardholderName"
                            onChange={handleInputChange}
                            required
                        />
                    ) : null}
                    <div className="FormRow elements-style">
                        <PaymentElement
                            onChange={(e) => {
                                setPaymentType(e.value.type);
                            }}
                        />
                    </div>
                </fieldset>
                <button
                    className="elements-style-background"
                    type="submit"
                    disabled={
                        !["initial", "succeeded", "error"].includes(payment.status) ||
                        !stripe
                    }
                >
                    Pay
                </button>
            </Form>
            <PaymentStatus status={payment.status}/>
        </>
    );
}

export const ElementsForm: FC<PaymentFormProps> = ({amount, sessionId}) => {
    return (
    <Elements
        stripe={getStripe()}
        options={{
            appearance: {
                    variables: {
                        colorIcon: "#6772e5",
                        fontFamily: "Roboto, Open Sans, Segoe UI, sans-serif",
                    },
                },
                currency: 'eur',
                mode: "payment",
                amount: amount*100,
            }}
        >
            <CheckoutForm amount={amount} sessionId={sessionId}/>
        </Elements>
    )
}